﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RemoteControl
{
    class Program
    {
        static void Main(string[] args)
        {
            //家中的电器
            Fan fan=new Fan();
            Light light=new Light();
            Heater heater=new Heater();

            //电器分别对应的命令
            FanOffCommand fanOffCommand=new FanOffCommand(fan);
            FanOnCommand fanOnCommand=new FanOnCommand(fan);
            LightOnCommand lightOnCommand=new LightOnCommand(light);
            LightOffCommand lightOffCommand=new LightOffCommand(light);
            HeaterOnCommand heaterOnCommand=new HeaterOnCommand(heater);
            HeaterOffCommand heaterOffCommand=new HeaterOffCommand(heater);
            RemoteControl remoteControl = new RemoteControl();

            //设置遥控器
            remoteControl.SetCommand(0, fanOnCommand, fanOffCommand);
            remoteControl.SetCommand(1, lightOnCommand, lightOffCommand);
            remoteControl.SetCommand(2, heaterOnCommand, heaterOffCommand);
            //分别测试遥控器的命令
            remoteControl.OnButtonWasPressed(1);
            remoteControl.OffButtonWasPressed(1);
            remoteControl.OnButtonWasPressed(0);
            remoteControl.OffButtonWasPressed(0);
            remoteControl.BackButtonWasPressed();

            //测试多个命令
            MutlipleCommand mutlipleCommand = new MutlipleCommand(new ICommand[] { new LightOffCommand(light),new FanOffCommand(fan)});
            mutlipleCommand.Excute();
            mutlipleCommand.Undo();
            Console.ReadKey();
        }
    }

    /// <summary>
    /// 风扇类
    /// </summary>
   public class Fan
    {
        public void FanOn()
        {
            Console.WriteLine("风扇开了");
        }
        public void FanOff()
        {
            Console.WriteLine("风扇关了");
        }
    }
    /// <summary>
    /// 灯类
    /// </summary>
   public class Light
   {
       public void LightOn()
       {
           Console.WriteLine("灯亮了");
       }
       public void LightOff()
       {
           Console.WriteLine("灯灭了");
       }
   }
    /// <summary>
    /// 热水器类
    /// </summary>
   public class Heater
   {
       public void HeaterOn()
       {
           Console.WriteLine("加热中");
       }
       public void HeaterOff()
       {
           Console.WriteLine("停止加热");
       }
   }

    /// <summary>
    /// 命令接口
    /// </summary>
   public interface ICommand
   {
       void Excute();
       void Undo();
   }

   public class FanOnCommand : ICommand
   {
       Fan fan;
       public FanOnCommand(Fan fan)
       {
           this.fan = fan;
       }
        public void Excute()
        {
 	       this.fan.FanOn();
        }


        public void Undo()
        {
            this.fan.FanOff();
        }
   }

   public class FanOffCommand : ICommand
   {
       Fan fan;
       public FanOffCommand(Fan fan)
       {
           this.fan = fan;
       }
        public void Excute()
        {
 	       this.fan.FanOff();
        }



        public void Undo()
        {
            this.fan.FanOn();
        }
   }

   public class LightOnCommand : ICommand
   {
       Light light;
       public LightOnCommand(Light light)
       {
           this.light = light; 
       }
       public void Excute()
       {
           light.LightOn();
       }
       /// <summary>
       /// 调用命令的反命令
       /// </summary>
       public void Undo()
       {
           light.LightOff();
       }
   }

   public class LightOffCommand : ICommand
   {
       Light light;
       public LightOffCommand(Light light)
       {
           this.light = light;
       }
       public void Excute()
       {
           this.light.LightOff();
       }


       public void Undo()
       {
           this.light.LightOn();
       }
   }

   public class HeaterOnCommand : ICommand
   {
       Heater heater;
       public HeaterOnCommand(Heater heater)
       {
           this.heater = heater;
       }
       public void Excute()
       {
           this.heater.HeaterOn();
       }


       public void Undo()
       {
           this.heater.HeaterOff();
       }
   }

   public class HeaterOffCommand : ICommand
   { 
       Heater heater;
       public HeaterOffCommand(Heater heater)
       {
           this.heater = heater;
       }
       public void Excute()
       {
           this.heater.HeaterOff();
       }


       public void Undo()
       {
           this.heater.HeaterOn();
       }
   }

   public class NoCommand : ICommand
   {
       public void Excute()
       { }



       public void Undo()
       {
           throw new NotImplementedException();
       }
   }

   public class RemoteControl
   {
      private ICommand[] onCommands;
      private ICommand[] offCommands;
       /// <summary>
       /// 撤销命令
       /// </summary>
       private ICommand backCommand;
      public RemoteControl()
      {
         ICommand noCommand=new NoCommand();
          onCommands = new ICommand[4];
          offCommands = new ICommand[4];
          for (int i = 0; i < 4; i++)
          {
              onCommands[i] = noCommand;
              offCommands[i] = noCommand;
          }
          backCommand=noCommand;
      }

      public void SetCommand(int slot, ICommand onCommand, ICommand offCommand)
      {
          onCommands[slot] = onCommand;
          offCommands[slot] = offCommand;
      }
      public void OnButtonWasPressed(int slot)
      {
          onCommands[slot].Excute();
          backCommand=onCommands[slot];
      }
      public void OffButtonWasPressed(int slot)
      {
          offCommands[slot].Excute();
          backCommand = offCommands[slot];
      }
      public void BackButtonWasPressed()
      {
          backCommand.Undo();
      }
   }

   public class MutlipleCommand : ICommand
   {
       ICommand[] commands;
       ICommand[] backCommands;
       public MutlipleCommand(ICommand[] commands)
       {
           this.commands = commands;
           backCommands = new ICommand[commands.Length];
       }


       public void Excute()
       {
           for (int i = 0; i < commands.Length; i++)
			{
			  commands[i].Excute();
              backCommands[i] = commands[i];
			}
           
       }

       public void Undo()
       {
           for (int i = 0; i < commands.Length; i++)
           {
               backCommands[i].Undo();
           }
       }
   }
}
    